@@ -75,6 +75,10 @@ class StorageTests(object):
         assert etag2 == etag
 
     def test_upload_already_existing(self, s, get_item):
+        if getattr(self, 'dav_server', '') == 'baikal':
+            # https://github.com/untitaker/vdirsyncer/issues/160
+            pytest.xfail(reason='Baikal uses an old version of SabreDAV.')
+
         item = get_item()
         s.upload(item)
         with pytest.raises(exceptions.PreconditionFailed):
@@ -97,11 +101,19 @@ class StorageTests(object):
         assert_item_equals(s.get(href)[0], new_item)
 
     def test_update_nonexisting(self, s, get_item):
+        if getattr(self, 'dav_server', '') == 'baikal':
+            # https://github.com/untitaker/vdirsyncer/issues/160
+            pytest.xfail(reason='Baikal uses an old version of SabreDAV.')
+
         item = get_item()
         with pytest.raises(exceptions.PreconditionFailed):
             s.update('huehue', item, '"123"')
 
     def test_wrong_etag(self, s, get_item):
+        if getattr(self, 'dav_server', '') == 'baikal':
+            # https://github.com/untitaker/vdirsyncer/issues/160
+            pytest.xfail(reason='Baikal uses an old version of SabreDAV.')
+
         item = get_item()
         href, etag = s.upload(item)
         with pytest.raises(exceptions.PreconditionFailed):
@@ -187,3 +199,10 @@ class StorageTests(object):
         # Can't do stronger assertion because of radicale, which needs a
         # fileextension to guess the collection type.
         assert 'test2' in s.collection
+
+    def test_case_sensitive_uids(self, s, get_item):
+        s.upload(get_item(uid='A' * 42))
+        s.upload(get_item(uid='a' * 42))
+        items = list(href for href, etag in s.list())
+        assert len(items) == 2
+        assert len(set(items)) == 2
