@@ -16,7 +16,6 @@ import vdirsyncer.storage.http
 from vdirsyncer.storage.singlefile import SingleFileStorage
 
 from . import StorageTests
-from .. import assert_item_equals
 
 
 class CombinedStorage(Storage):
@@ -54,7 +53,7 @@ class TestHttpStorage(StorageTests):
 
     @pytest.fixture(autouse=True)
     def setup_tmpdir(self, tmpdir, monkeypatch):
-        self.tmpfile = str(tmpdir.join('collection.txt'))
+        self.tmpfile = str(tmpdir.ensure('collection.txt'))
 
         def _request(method, url, *args, **kwargs):
             assert method == 'GET'
@@ -81,17 +80,3 @@ class TestHttpStorage(StorageTests):
             return {'url': 'http://localhost:123/collection.txt',
                     'path': self.tmpfile}
         return inner
-
-    def test_update(self, s, get_item):
-        '''The original testcase tries to fetch with the old href. But this
-        storage doesn't have real hrefs, so the href might change if the
-        underlying UID changes. '''
-
-        item = get_item()
-        href, etag = s.upload(item)
-        assert_item_equals(s.get(href)[0], item)
-
-        new_item = get_item()
-        s.update(href, new_item, etag)
-        ((new_href, new_etag),) = s.list()
-        assert_item_equals(s.get(new_href)[0], new_item)
