@@ -55,7 +55,7 @@ def test_load_config(monkeypatch):
         type = filesystem
         path = /tmp/contacts/
         fileext = .vcf
-        yesno = off
+        yesno = false
         number = 42
 
         [storage bob_b]
@@ -126,6 +126,9 @@ def test_simple_run(tmpdir, runner):
     fileext = .txt
     ''').format(str(tmpdir)))
 
+    tmpdir.mkdir('path_a')
+    tmpdir.mkdir('path_b')
+
     result = runner.invoke(['sync'])
     assert not result.exception
 
@@ -152,6 +155,9 @@ def test_empty_storage(tmpdir, runner):
     fileext = .txt
     ''').format(str(tmpdir)))
 
+    tmpdir.mkdir('path_a')
+    tmpdir.mkdir('path_b')
+
     result = runner.invoke(['sync'])
     assert not result.exception
 
@@ -274,7 +280,10 @@ def test_collections_cache_invalidation(tmpdir, runner):
 
     foo = tmpdir.mkdir('foo')
     bar = tmpdir.mkdir('bar')
-    foo.mkdir('a').join('itemone.txt').write('UID:itemone')
+    for x in 'abc':
+        foo.mkdir(x)
+        bar.mkdir(x)
+    foo.join('a/itemone.txt').write('UID:itemone')
 
     result = runner.invoke(['sync'])
     assert not result.exception
@@ -302,6 +311,8 @@ def test_collections_cache_invalidation(tmpdir, runner):
 
     tmpdir.join('status').remove()
     bar2 = tmpdir.mkdir('bar2')
+    for x in 'abc':
+        bar2.mkdir(x)
     result = runner.invoke(['sync'])
     assert not result.exception
 
@@ -329,8 +340,10 @@ def test_invalid_pairs_as_cli_arg(tmpdir, runner):
     collections = ["a", "b", "c"]
     ''').format(str(tmpdir)))
 
-    tmpdir.mkdir('foo')
-    tmpdir.mkdir('bar')
+    for base in ('foo', 'bar'):
+        base = tmpdir.mkdir(base)
+        for c in 'abc':
+            base.mkdir(c)
 
     result = runner.invoke(['sync', 'foobar/d'])
     assert result.exception
@@ -356,11 +369,12 @@ def test_discover_command(tmpdir, runner):
     ''').format(str(tmpdir)))
 
     foo = tmpdir.mkdir('foo')
-    tmpdir.mkdir('bar')
+    bar = tmpdir.mkdir('bar')
 
-    foo.mkdir('a')
-    foo.mkdir('b')
-    foo.mkdir('c')
+    for x in 'abc':
+        foo.mkdir(x)
+        bar.mkdir(x)
+    bar.mkdir('d')
 
     result = runner.invoke(['sync'])
     assert not result.exception
@@ -369,10 +383,14 @@ def test_discover_command(tmpdir, runner):
     assert 'Syncing foobar/a' in lines
     assert 'Syncing foobar/b' in lines
     assert 'Syncing foobar/c' in lines
+    assert 'Syncing foobar/d' not in lines
 
     foo.mkdir('d')
     result = runner.invoke(['sync'])
     assert not result.exception
+    assert 'Syncing foobar/a' in lines
+    assert 'Syncing foobar/b' in lines
+    assert 'Syncing foobar/c' in lines
     assert 'Syncing foobar/d' not in result.output
 
     result = runner.invoke(['discover'])
@@ -380,6 +398,9 @@ def test_discover_command(tmpdir, runner):
 
     result = runner.invoke(['sync'])
     assert not result.exception
+    assert 'Syncing foobar/a' in lines
+    assert 'Syncing foobar/b' in lines
+    assert 'Syncing foobar/c' in lines
     assert 'Syncing foobar/d' in result.output
 
 
@@ -403,12 +424,12 @@ def test_multiple_pairs(tmpdir, runner):
     runner.write_with_general(''.join(get_cfg()))
 
     result = runner.invoke(['sync'])
-    assert sorted(result.output.splitlines()) == [
+    assert set(result.output.splitlines()) > set([
         'Discovering collections for pair bambaz',
         'Discovering collections for pair foobar',
         'Syncing bambaz',
         'Syncing foobar',
-    ]
+    ])
 
 
 def test_invalid_collections_arg(tmpdir, runner):
@@ -437,45 +458,62 @@ def test_invalid_collections_arg(tmpdir, runner):
     )
 
 
-def test_parse_config_value():
-    x = cli.utils.parse_config_value
-    with pytest.raises(ValueError):
-        x('123  # comment!')
+def test_create_collections(tmpdir, runner):
+    runner.write_with_general(dedent('''
+    [pair foobar]
+    a = foo
+    b = bar
+    collections = ["a", "b", "c"]
 
-    assert x('"123  # comment!"') == '123  # comment!'
-    assert x('True') is True
-    assert x('False') is False
-    assert x('Yes') is True
-    assert x('3.14') == 3.14
-    assert x('') == ''
-    assert x('""') == ''
+    [storage foo]
+    type = filesystem
+    path = {base}/foo/
+    fileext = .txt
+
+    [storage bar]
+    type = filesystem
+    path = {base}/bar/
+    fileext = .txt
+    '''.format(base=str(tmpdir))))
+
+    result = runner.invoke(['sync'])
+    assert result.exception
+    entries = set(x.basename for x in tmpdir.listdir())
+    assert 'foo' not in entries and 'bar' not in entries
+
+    result = runner.invoke(['sync'], input='y\n' * 6)
+    assert not result.exception
+    assert \
+        set(x.basename for x in tmpdir.join('foo').listdir()) == \
+        set(x.basename for x in tmpdir.join('bar').listdir()) == \
+        set('abc')
 
 
-def test_parse_options():
-    o = {
-        'foo': 'yes',
-        'hah': 'true',
-        'bar': '',
-        'baz': 'whatever',
-        'bam': '123',
-        'asd': 'off'
-    }
+def test_parse_config_value(capsys):
+    invalid = object()
 
-    a = dict(cli.utils.parse_options(o.items()))
+    def x(s):
+        try:
+            rv = cli.utils.parse_config_value(s)
+        except ValueError:
+            return invalid
+        else:
+            warnings = capsys.readouterr()[1]
+            return rv, len(warnings.splitlines())
 
-    expected = {
-        'foo': True,
-        'hah': True,
-        'bar': '',
-        'baz': 'whatever',
-        'bam': 123,
-        'asd': False
-    }
+    assert x('123  # comment!') is invalid
 
-    assert a == expected
+    assert x('True') == ('True', 1)
+    assert x('False') == ('False', 1)
+    assert x('Yes') == ('Yes', 1)
+    assert x('None') == ('None', 1)
+    assert x('"True"') == ('True', 0)
+    assert x('"False"') == ('False', 0)
 
-    for key in a:
-        # Yes, we want a very strong typecheck here, because we actually have
-        # to differentiate between bool and int, and in Python 2, bool is a
-        # subclass of int.
-        assert a[key].__class__ is expected[key].__class__
+    assert x('"123  # comment!"') == ('123  # comment!', 0)
+    assert x('true') == (True, 0)
+    assert x('false') == (False, 0)
+    assert x('null') == (None, 0)
+    assert x('3.14') == (3.14, 0)
+    assert x('') == ('', 0)
+    assert x('""') == ('', 0)
