@@ -16,7 +16,7 @@ import sys
 import threading
 from itertools import chain
 
-from .. import DOCS_HOME, PROJECT_HOME, log
+from .. import DOCS_HOME, PROJECT_HOME, exceptions, log
 from ..doubleclick import click
 from ..storage import storage_names
 from ..sync import StorageEmpty, SyncConflict
@@ -89,7 +89,7 @@ def validate_section_name(name, section_type):
     invalid = set(name) - SECTION_NAME_CHARS
     if invalid:
         chars_display = ''.join(sorted(SECTION_NAME_CHARS))
-        raise CliError('The {}-section {!r} contains invalid characters. Only '
+        raise CliError('The {}-section "{}" contains invalid characters. Only '
                        'the following characters are allowed for storage and '
                        'pair names:\n{}'.format(section_type, name,
                                                 chars_display))
@@ -167,22 +167,27 @@ def _get_coll(pair_name, storage_name, collection, discovered, config):
     try:
         return discovered[collection]
     except KeyError:
+        return _handle_collection_not_found(config, collection)
+
+
+def _handle_collection_not_found(config, collection):
+    storage_name = config.get('instance_name', None)
+    cli_logger.error('No collection {} found for storage {}.'
+                     .format(collection, storage_name))
+    if click.confirm('Should vdirsyncer attempt to create it?'):
         storage_type = config['type']
         cls, config = storage_class_from_config(config)
         try:
-            args = cls.join_collection(collection=collection, **config)
+            args = cls.create_collection(collection=collection, **config)
             args['type'] = storage_type
             return args
         except NotImplementedError as e:
             cli_logger.error(e)
-            raise CliError(
-                '{pair}: Unable to find or create collection {collection!r} '
-                'for storage {storage!r}. A same-named collection was found '
-                'for the other storage, and vdirsyncer is configured to '
-                'synchronize these two collections. Please create the '
-                'collection yourself.'
-                .format(pair=pair_name, collection=collection,
-                        storage=storage_name))
+
+    raise CliError('Unable to find or create collection "{collection}" for '
+                   'storage "{storage}". Please create the collection '
+                   'yourself.'.format(collection=collection,
+                                      storage=storage_name))
 
 
 def _collections_for_pair_impl(status_path, name_a, name_b, pair_name,
@@ -353,19 +358,25 @@ def storage_class_from_config(config):
     return cls, config
 
 
-def storage_instance_from_config(config):
+def storage_instance_from_config(config, create=True):
     '''
     :param config: A configuration dictionary to pass as kwargs to the class
         corresponding to config['type']
-    :param description: A name for the storage for debugging purposes
     '''
 
-    cls, config = storage_class_from_config(config)
+    cls, new_config = storage_class_from_config(config)
 
     try:
-        return cls(**config)
+        return cls(**new_config)
+    except exceptions.CollectionNotFound as e:
+        cli_logger.error(str(e))
+        if create:
+            _handle_collection_not_found(config, None)
+            return storage_instance_from_config(config, create=False)
+        else:
+            raise
     except Exception:
-        handle_storage_init_error(cls, config)
+        return handle_storage_init_error(cls, new_config)
 
 
 def handle_storage_init_error(cls, config):
@@ -460,28 +471,17 @@ def parse_config_value(value):
     try:
         return json.loads(value)
     except ValueError:
-        rv = value
+        pass
 
-    if value.lower() in ('on', 'true', 'yes'):
-        cli_logger.warning(
-            '{} is deprecated for the config, please use true.\n'
-            'The old form will be removed in 0.4.0.'
-            .format(value)
-        )
-        return True
-    if value.lower() in ('off', 'false', 'no'):
-        cli_logger.warning(
-            '{} is deprecated for the config, please use false.\n'
-            'The old form will be removed in 0.4.0.'
-            .format(value)
-        )
-        return False
-    if value.lower() == 'none':
-        cli_logger.warning(
-            'None is deprecated for the config, please use null.\n'
-            'The old form will be removed in 0.4.0.'
-        )
-        return None
+    for wrong, right in [
+        (('on', 'yes'), 'true'),
+        (('off', 'no'), 'false'),
+        (('none',), 'null')
+    ]:
+        if value.lower() in wrong + (right,):
+            cli_logger.warning('You probably meant {} instead of "{}", which '
+                               'will now be interpreted as a literal string.'
+                               .format(right, value))
 
     if '#' in value:
         raise ValueError('Invalid value:{}\n'
@@ -495,9 +495,9 @@ def parse_config_value(value):
         #
         # foo = bar
         #  # my comment
-        raise ValueError('No multiline-values allowed:\n{!r}'.format(value))
+        raise ValueError('No multiline-values allowed:\n{}'.format(value))
 
-    return rv
+    return value
 
 
 def parse_options(items, section=None):
@@ -505,7 +505,7 @@ def parse_options(items, section=None):
         try:
             yield key, parse_config_value(value)
         except ValueError as e:
-            raise ValueError('Section {!r}, option {!r}: {}'
+            raise ValueError('Section "{}", option "{}": {}'
                              .format(section, key, e))
 
 
