@@ -23,6 +23,10 @@ class Error(Exception):
         super(Error, self).__init__(*args)
 
 
+class CollectionNotFound(Error):
+    '''Collection not found'''
+
+
 class PreconditionFailed(Error):
     '''
       - The item doesn't exist although it should
@@ -48,3 +52,7 @@ class WrongEtagError(PreconditionFailed):
 
 class ReadOnlyError(Error):
     '''Storage is read-only.'''
+
+
+class InvalidResponse(Error, ValueError):
+    '''The backend returned an invalid result.'''
