@@ -7,13 +7,11 @@
     :license: MIT, see LICENSE for more details.
 '''
 import datetime
-import functools
-import itertools
 
 from lxml import etree
 
 from requests import session as requests_session
-from requests.exceptions import RequestException
+from requests.exceptions import HTTPError
 
 from .base import Item, Storage
 from .http import USERAGENT, prepare_auth, prepare_verify
@@ -43,12 +41,23 @@ def _decode_href(x):
     return utils.compat.urlunquote(x)
 
 
+class InvalidXMLResponse(exceptions.InvalidResponse):
+    pass
+
+
 def _parse_xml(content):
     try:
         return etree.XML(content)
     except etree.Error as e:
-        raise ValueError('Invalid XML encountered: {}\n'
-                         'Double-check the URLs in your config.'.format(e))
+        raise InvalidXMLResponse('Invalid XML encountered: {}\n'
+                                 'Double-check the URLs in your config.'
+                                 .format(e))
+
+
+def _merge_xml(items):
+    rv = items[0]
+    rv.extend(items[1:])
+    return rv
 
 
 def _fuzzy_matches_mimetype(strict, weak):
@@ -65,24 +74,12 @@ def _fuzzy_matches_mimetype(strict, weak):
     return False
 
 
-def _catch_generator_exceptions(f):
-    @functools.wraps(f)
-    def inner(*args, **kwargs):
-        try:
-            for x in f(*args, **kwargs):
-                yield x
-        except (RequestException, exceptions.Error):
-            pass
-    return inner
+def _get_collection_from_url(url):
+    _, collection = url.rstrip('/').rsplit('/', 1)
+    return collection
 
 
 class Discover(object):
-    # Another one of Radicale's specialties: Discovery is broken (returning
-    # completely wrong URLs at every stage) as of version 0.9.
-    # https://github.com/Kozea/Radicale/issues/196
-    #
-    # So we just brute-force a lot of paths here.
-
     _resourcetype = None
     _homeset_xml = None
     _homeset_tag = None
@@ -99,18 +96,21 @@ class Discover(object):
     def __init__(self, session):
         self.session = session
 
-    def find_principal(self):
-        for server in self.find_dav():
-            for x in list(self._find_principal(server)) or ['']:
-                yield x
+    def find_dav(self):
+        try:
+            response = self.session.request('GET', self._well_known_uri,
+                                            allow_redirects=False,
+                                            is_subpath=False)
+            return response.headers.get('Location', '')
+        except (HTTPError, exceptions.Error):
+            # The user might not have well-known URLs set up and instead points
+            # vdirsyncer directly to the DAV server.
+            dav_logger.debug('Server does not support well-known URIs.')
+            return ''
 
-    @_catch_generator_exceptions
-    def _find_principal(self, url):
-        """tries to find the principal URL of the user
-        :returns: iterable (but should be only of element) of urls
-        :rtype: iterable(unicode)
-
-        """
+    def find_principal(self, url=None):
+        if url is None:
+            url = self.find_dav()
         headers = self.session.get_default_headers()
         headers['Depth'] = 0
         body = """
@@ -123,82 +123,64 @@ class Discover(object):
         response = self.session.request('PROPFIND', url, headers=headers,
                                         data=body, is_subpath=False)
         root = _parse_xml(response.content)
+        rv = root.find('.//{DAV:}current-user-principal/{DAV:}href')
+        if rv is None:
+            raise InvalidXMLResponse()
+        return rv.text
 
-        for element in root.iter('{*}current-user-principal'):
-            for principal in element.iter():  # should be only one
-                if principal.tag.endswith('href'):
-                    yield principal.text
-
-    def find_dav(self):
-        return list(self._find_dav()) or ['']
-
-    @_catch_generator_exceptions
-    def _find_dav(self):
-        response = self.session.request('GET', self._well_known_uri,
-                                        allow_redirects=False,
-                                        is_subpath=False)
-        yield response.headers.get('Location', '')
-
-    def discover(self):
-        """discover all the user's CalDAV or CardDAV collections on the server
-        :returns: a list of the user's collections (as urls)
-        :rtype: list(unicode)
-        """
-
-        done = set()
-        for collection in self.find_collections():
-            collection['href'] = href = \
-                utils.urlparse.urljoin(self.session.url, collection['href'])
-            if href not in done:
-                done.add(href)
-                yield collection
-
-    def find_homes(self):
-        for principal in self.find_principal():
-            for x in self._find_homes(principal):
-                yield x
-
-    @_catch_generator_exceptions
-    def _find_homes(self, principal):
+    def find_home(self, url=None):
+        if url is None:
+            url = self.find_principal()
         headers = self.session.get_default_headers()
         headers['Depth'] = 0
-        response = self.session.request('PROPFIND', principal, headers=headers,
+        response = self.session.request('PROPFIND', url,
+                                        headers=headers,
                                         data=self._homeset_xml,
                                         is_subpath=False)
 
         root = etree.fromstring(response.content)
-        for element in root.iter(self._homeset_tag):
-            for homeset in element.iter():
-                if homeset.tag.endswith('href'):
-                    yield homeset.text
-
-    def find_collections(self):
-        for home in itertools.chain(self.find_homes(), ['']):
-            for x in self._find_collections(home):
-                yield x
-
-    @_catch_generator_exceptions
-    def _find_collections(self, home):
-        """find all CalDAV collections under `home`"""
+        # Better don't do string formatting here, because of XML namespaces
+        rv = root.find('.//' + self._homeset_tag + '/{*}href')
+        if rv is None:
+            raise InvalidXMLResponse()
+        return rv.text
 
+    def find_collections(self, url=None):
+        if url is None:
+            url = self.find_home()
         headers = self.session.get_default_headers()
         headers['Depth'] = 1
-        response = self.session.request('PROPFIND', home, headers=headers,
+        response = self.session.request('PROPFIND', url, headers=headers,
                                         data=self._collection_xml,
                                         is_subpath=False)
         root = _parse_xml(response.content)
-        for response in root.iter('{*}response'):
-            prop = response.find('{*}propstat/{*}prop')
-            if prop.find('{*}resourcetype/{*}' + self._resourcetype) is None:
+        done = set()
+        for response in root.findall('{DAV:}response'):
+            props = _merge_xml(response.findall('{*}propstat/{*}prop'))
+            if props.find('{*}resourcetype/{*}' + self._resourcetype) is None:
                 continue
 
-            displayname = prop.find('{*}displayname')
-            collection = {
-                'href': response.find('{*}href').text,
-                'displayname': '' if displayname is None else displayname.text
-            }
+            displayname = getattr(props.find('{*}displayname'), 'text', '')
+            href = response.find('{*}href')
+            if href is None:
+                raise InvalidXMLResponse()
+            href = utils.urlparse.urljoin(self.session.url, href.text)
+            if href not in done:
+                done.add(href)
+                yield {'href': href, 'displayname': displayname}
 
-            yield collection
+    def discover(self):
+        for x in self.find_collections():
+            yield x
+
+        # Another one of Radicale's specialties: Discovery is broken (returning
+        # completely wrong URLs at every stage) as of version 0.9.
+        # https://github.com/Kozea/Radicale/issues/196
+        try:
+            for x in self.find_collections(''):
+                yield x
+        except (InvalidXMLResponse, HTTPError, exceptions.Error):
+            pass
 
 
 class CalDiscover(Discover):
@@ -334,14 +316,16 @@ class DavStorage(Storage):
         d = cls.discovery_class(cls._get_session(**kwargs))
         for c in d.discover():
             url = c['href']
-            _, collection = url.rstrip('/').rsplit('/', 1)
+            collection = _get_collection_from_url(url)
             storage_args = dict(kwargs)
             storage_args.update({'url': url, 'collection': collection,
                                  'collection_human': c['displayname']})
             yield storage_args
 
     @classmethod
-    def join_collection(cls, collection, **kwargs):
+    def create_collection(cls, collection, **kwargs):
+        if collection is None:
+            collection = _get_collection_from_url(kwargs['url'])
         session = cls._get_session(**kwargs)
         d = cls.discovery_class(session)
 
@@ -349,17 +333,13 @@ class DavStorage(Storage):
             if c['collection'] == collection:
                 return c
 
-        homes = list(d.find_homes())
-        if len(homes) != 1:
-            raise NotImplementedError('Not sure where to create {r!}, {} '
-                                      'homeset-URLs found (need exactly 1).'
-                                      .format(collection, len(homes)))
+        home = d.find_home()
+        collection_url = '{}/{}'.format(home.rstrip('/'), collection)
 
         try:
-            collection_url = '{}/{}'.format(homes[0].rstrip('/'), collection)
             response = d.session.request('MKCOL', collection_url,
                                          is_subpath=False)
-        except RequestException as e:
+        except HTTPError as e:
             raise NotImplementedError(e)
         else:
             rv = dict(kwargs)
@@ -405,17 +385,17 @@ class DavStorage(Storage):
         rv = []
         hrefs_left = set(hrefs)
         for href, etag, prop in self._parse_prop_responses(root):
-            try:
-                raw = prop.find(self.get_multi_data_query).text
-            except AttributeError:
+            raw = prop.find(self.get_multi_data_query)
+            if raw is None:
                 dav_logger.warning('Skipping {}, the item content is missing.'
                                    .format(href))
                 continue
-
-            if isinstance(raw, bytes):
-                raw = raw.decode(response.encoding)
-            if isinstance(etag, bytes):
-                etag = etag.decode(response.encoding)
+            else:
+                raw = raw.text
+                if isinstance(raw, bytes):
+                    raw = raw.decode(response.encoding)
+                if isinstance(etag, bytes):
+                    etag = etag.decode(response.encoding)
 
             try:
                 hrefs_left.remove(href)
@@ -453,10 +433,9 @@ class DavStorage(Storage):
         return href, etag
 
     def update(self, href, item, etag):
-        href = self._normalize_href(href)
         if etag is None:
             raise ValueError('etag must be given and must not be None.')
-        href, etag = self._put(href, item, etag)
+        href, etag = self._put(self._normalize_href(href), item, etag)
         return etag
 
     def upload(self, item):
@@ -479,13 +458,12 @@ class DavStorage(Storage):
     def _parse_prop_responses(self, root, decoding_rounds=1):
         hrefs = set()
         for response in root.iter('{DAV:}response'):
-            try:
-                href = response.find('{DAV:}href').text
-            except AttributeError:
+            href = response.find('{DAV:}href')
+            if href is None:
                 dav_logger.error('Skipping response, href is missing.')
                 continue
 
-            href = self._normalize_href(href)
+            href = self._normalize_href(href.text)
             for i in range(decoding_rounds):
                 href = _decode_href(href)
 
@@ -499,21 +477,25 @@ class DavStorage(Storage):
                                    .format(href))
                 continue
 
-            try:
-                prop = response.find('{DAV:}propstat/{DAV:}prop')
-                contenttype = prop.find('{DAV:}getcontenttype')
-                etag = prop.find('{DAV:}getetag').text
-            except AttributeError:
+            props = response.findall('{DAV:}propstat/{DAV:}prop')
+            if not props:
                 dav_logger.warning('Skipping {!r}, properties are missing.'
                                    .format(href))
                 continue
+            else:
+                props = _merge_xml(props)
 
-            is_collection = prop.find('{DAV:}resourcetype/{DAV:}collection')
-            if is_collection is not None:
+            if props.find('{DAV:}resourcetype/{DAV:}collection') is not None:
                 dav_logger.debug('Skipping {!r}, is collection.'.format(href))
                 continue
 
-            contenttype = getattr(contenttype, 'text', None)
+            etag = getattr(props.find('{DAV:}getetag'), 'text', '')
+            if not etag:
+                dav_logger.warning('Skipping {!r}, etag property is missing.'
+                                   .format(href))
+
+            contenttype = getattr(props.find('{DAV:}getcontenttype'),
+                                  'text', None)
             if not self._is_item_mimetype(contenttype):
                 dav_logger.debug('Skipping {!r}, {!r} != {!r}.'
                                  .format(href, contenttype,
@@ -521,7 +503,7 @@ class DavStorage(Storage):
                 continue
 
             hrefs.add(href)
-            yield href, etag, prop
+            yield href, etag, props
 
 
 class CaldavStorage(DavStorage):
